
/**************************************************************************

			PACKAGE VOTACIONS

**************************************************************************/

CREATE OR REPLACE PACKAGE USUARITFC11.PKG_VOTACIONS AS

PROCEDURE PR_ALTA_VOTACIO  (
	p_titol 	   IN VARCHAR2,
  p_descripcio	   IN VARCHAR2,
	p_datainici	   IN DATE,
	p_datafi	   IN DATE,
	p_datapublicacio   IN DATE,
	p_president	   IN VARCHAR2,
	p_censglobal	   IN NUMBER,
	p_web		   IN VARCHAR2,
	RSP		   OUT VARCHAR2
);

PROCEDURE PR_BAIXA_VOTACIO (
	p_idvotacio	IN NUMBER,	
	RSP 		OUT VARCHAR2
);

PROCEDURE PR_MODIFICACIO_VOTACIO (
	p_idvotacio	      IN NUMBER,	
	p_titol 	      IN VARCHAR2,
        p_descripcio	      IN VARCHAR2,
	p_datainici	      IN DATE,
	p_datafi	      IN DATE,
	p_datapublicacio      IN DATE,
	p_president	      IN VARCHAR2,
	p_censglobal	      IN NUMBER,
	p_web		      IN VARCHAR2,
	p_totalciutadans      IN NUMBER,
	p_votstotals	      IN NUMBER,
	p_percentatgevotacio  IN NUMBER,
	RSP		      OUT VARCHAR2
);

PROCEDURE PR_ALTA_OPCIO  (
	p_idvotacio 	   IN NUMBER,
  p_opcio		   IN VARCHAR2,
	RSP		   OUT VARCHAR2
)
;

PROCEDURE PR_BAIXA_OPCIO  (
	p_idopcio   	IN NUMBER,
	RSP		OUT VARCHAR2
);


PROCEDURE PR_MODIFICACIO_OPCIO (
	p_idopcio	      IN NUMBER,
	p_idvotacio	      IN NUMBER,	
	p_opcio 	      IN VARCHAR2,
  p_votstotals	      IN NUMBER,
	p_percentatge	      IN NUMBER,
	RSP		      OUT VARCHAR2
);

END PKG_VOTACIONS;
/

CREATE OR REPLACE PACKAGE BODY USUARITFC11.PKG_VOTACIONS AS

/********************************************************
*							*
*		MANTENIMENT VOTACIONS			*
*							*
********************************************************/

/*******************************************************************************************************

NOM PROCEDIMENT: P_ALTA_VOTACIO

FUNCIONALITAT:
Aquest procediment permet donar dalta una nova votaci a la Base de dades, si prviament supera els controls de:
   Tots els camps obligatoris continguin valor.
   El registre no shagi inserit amb un altre identificador.
   Amb els camps de tipus data, shan de tenir en compte els segents controls:
	- Sha de verificar que la data final ha de ser posterior a la data dinici.
	- Sha de verificar que la data de publicaci ha de ser com a mnim 7 dies abans que la data dinici de la votaci.

PARAMETRES

ENTRADA
Obligatoris:
	p_titol	Ttol de la votaci
	p_descripcio	Descripci de la votaci
	p_datainici	Data dinici de la votaci
	p_datafi 	Data fi de la votaci
	p_datapublicaci	Data de publicaci de la votaci
	p_president	Nom del president de la mesa electoral
	p_censglobal	Cens global de la votaci
Opcionals:
	p_web	Adrea web opcional

SORTIDA
Obligatoris:
	RSP	

PRECONDICI: No existeix la votaci a crear.
POSTCONDICI: Es dona dalta la votaci.

RETORNA 
   SI TOT CORRECTE
	OK si loperaci sha realitzat amb xit
   SI ES PRODUEIX UNA EXCEPCI
	ERROR: La votaci ja existeix a la Base de Dades.
	ERROR: Els camps obligatoris no poden ser nul.
	ERROR: La data final no pot ser anterior a la data dinici.
	ERROR: La data de publicaci ha de ser, com a mnim, 7 dies abans que la data dinici.
	ERROR: Tipus derror. En qualsevol altre cas.

*******************************************************************************************************/

PROCEDURE PR_ALTA_VOTACIO  (
	p_titol 	   IN VARCHAR2,
        p_descripcio	   IN VARCHAR2,
	p_datainici	   IN DATE,
	p_datafi	   IN DATE,
	p_datapublicacio   IN DATE,
	p_president	   IN VARCHAR2,
	p_censglobal	   IN NUMBER,
	p_web		   IN VARCHAR2,
	RSP		   OUT VARCHAR2
)

IS

E_VOTACIO_EXISTEIX	EXCEPTION;
E_CAMPS_NUL		EXCEPTION;
E_DATAFINAL_ANTERIOR	EXCEPTION;
E_DATAPUBLICACIO_MIN	EXCEPTION;

existeix	VARCHAR2(1);
dies		NUMBER;
p_entrada	VARCHAR2(1000);
error_num	NUMBER;
error_missatge	VARCHAR2(100);

BEGIN

   --S'emmagatzema els parmetres d'entrada a la variable p_entrada

   p_entrada := 'P_TITOL: ' ||p_titol||', P_DESCRIPCIO: '||p_descripcio||', P_DATAINICI: '||p_datainici||
		', P_DATAFI: '||p_datafi||', P_DATAPUBLICACIO: '||p_datapublicacio||', P_PRESIDENT: '||p_president||
		', P_CENSGLOBAL: '||p_censglobal||', P_WEB: '||p_web; 

   --Es verifica que tots els camps obligatoris contingin valor

   IF (TRIM(p_titol) IS NULL OR 
	TRIM(p_descripcio) IS NULL OR 
	TRIM(p_datainici) IS NULL OR 
	TRIM(p_datafi) IS NULL OR 
	TRIM(p_datapublicacio) IS NULL OR 
	TRIM(p_president) IS NULL OR 
	TRIM(p_censglobal) IS NULL)
   THEN
      RAISE E_CAMPS_NUL;
   END IF;

   --Es verifica que el registre no s'hagi insertat anteriorment amb altre identificador.

   BEGIN
   
   SELECT 'Y' INTO existeix 
   FROM VOTACIO 
   WHERE titol = p_titol AND ROWNUM = 1;

   RAISE E_VOTACIO_EXISTEIX;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
     NULL;
   END;

   --Es verifica que la data d'inici no sigui posterior a la data fi

   IF (p_datafi < p_datainici) THEN
	RAISE E_DATAFINAL_ANTERIOR;
   END IF;

   --Es verifica que la data publicaci ha de ser com a mnim 7 dies abans que la data dinici de la votaci

   dies := p_datainici - p_datapublicacio;
   IF (dies < 7) THEN
	RAISE E_DATAPUBLICACIO_MIN;
   END IF;

   --Si es pasa totes les proves de validaci, es realitzar l'inserci del nou registre a la Base de Dades.

   INSERT INTO VOTACIO (TITOL, DESCRIPCIO, DATAINICI, DATAFI, DATAPUBLICACIO, PRESIDENT, CENSGLOBAL, WEB)
   VALUES (p_titol, p_descripcio, p_datainici, p_datafi, p_datapublicacio, p_president, p_censglobal, p_web);
	
   RSP := 'OK';

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_VOTACIONS.PR_ALTA_VOTACIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   --Excepcions
EXCEPTION

 --La votacio existeix a la Base de Dades
   WHEN E_VOTACIO_EXISTEIX THEN RSP:= 'ERROR: La votaci ja existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_VOTACIONS.PR_ALTA_VOTACIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --Els camps obligatoris sn nuls
   WHEN E_CAMPS_NUL THEN RSP:= 'ERROR: Els camps obligatoris no poden ser nul'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_VOTACIONS.PR_ALTA_VOTACIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --La data final es anterior a la data inici
   WHEN E_DATAFINAL_ANTERIOR THEN RSP:= 'ERROR: La data final no pot ser anterior a la data dinici'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_VOTACIONS.PR_ALTA_VOTACIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT; 

 --La data publicaci posterior a 7 dies abans que la data inici
   WHEN E_DATAPUBLICACIO_MIN THEN 
	RSP:= 'ERROR: : La data de publicaci ha de ser, com a mnim, 7 dies abans que la data dinici'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_VOTACIONS.PR_ALTA_VOTACIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT; 

   WHEN OTHERS THEN
   error_num := SQLCODE;
   error_missatge := SUBSTR(SQLERRM, 1, 200);

   RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_VOTACIONS.PR_ALTA_VOTACIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

END PR_ALTA_VOTACIO;


/*******************************************************************************************************

NOM PROCEDIMENT: P_BAIXA_VOTACIO

FUNCIONALITAT:
Aquest procediment permet donar de baixa una votaci a la Base de dades, si prviament supera els controls de:
	Tots els camps obligatoris continguin valor.
	Existeix la votaci a la Base de Dades.

PARAMETRES:
ENTRADA
Obligatoris:
	p_idvotacio	Identificador de la votaci

SORTIDA
Obligatoris:
	RSP	

PRECONDICI: Existeix la votaci a donar de baixa.
POSTCONDICI: Es dona de baixa la votaci.

RETORNA
   SI TOT CORRECTE
	OK si loperaci sha realitzat amb xit
   SI ES PRODUEIX UNA EXCEPCI
	ERROR: La votaci a donar de baixa no existeix a la Base de Dades.
	ERROR: Lidentificador de la votaci no poden ser nul.
	ERROR: Tipus derror. En qualsevol altre cas.

*******************************************************************************************************/

PROCEDURE PR_BAIXA_VOTACIO (
	p_idvotacio	IN NUMBER,	
	RSP 		OUT VARCHAR2
)

IS

E_CAMPS_NUL		EXCEPTION;
E_VOTACIO_NOEXISTEIX	EXCEPTION;

existeix	VARCHAR2(1);
p_entrada	VARCHAR2(1000);
error_num	NUMBER;
error_missatge	VARCHAR2(100);

BEGIN

   --S'emmagatzema els parmetres d'entrada a la variable p_entrada

   p_entrada := 'P_IDVOTACIO: ' ||p_idvotacio;

   --Es verifica que tots els camps obligatoris contingin valor

   IF (TRIM(p_idvotacio) IS NULL) THEN
      RAISE E_CAMPS_NUL;
   END IF;

   --Es verifica que la votaci existeix a la Base de Dades

   BEGIN

   SELECT 'Y' INTO existeix 
   FROM VOTACIO 
   WHERE idvotacio = p_idvotacio AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
      RAISE E_VOTACIO_NOEXISTEIX;
   END;

   --Si es pasa totes les proves de validaci, es realitzar la baixa del registre a la Base de Dades.

   DELETE FROM VOTACIO
   WHERE idvotacio = p_idvotacio;

   RSP := 'OK';

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_VOTACIONS.PR_BAIXA_VOTACIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   --Excepcions
EXCEPTION

 --Els camps obligatoris sn nuls
   WHEN E_CAMPS_NUL THEN RSP:= 'ERROR: : Els camps obligatoris no poden ser nul'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_VOTACIONS.PR_BAIXA_VOTACIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --La votacio no existeix a la Base de Dades
   WHEN E_VOTACIO_NOEXISTEIX THEN RSP:= 'ERROR: : La votaci a donar de baixa no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_VOTACIONS.PR_BAIXA_VOTACIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   WHEN OTHERS THEN
   error_num := SQLCODE;
   error_missatge := SUBSTR(SQLERRM, 1, 200);

   RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_VOTACIONS.PR_BAIXA_VOTACIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

END PR_BAIXA_VOTACIO;


/*******************************************************************************************************

NOM PROCEDIMENT: P_MODIFICACIO_VOTACIONS

FUNCIONALITAT:

Aquest procediment permet modificar una votaci a la Base de dades, si prviament supera els controls de:
   La votaci existeix a la Base de Dades.
   Tots els camps obligatoris continguin valor.
   Si es vol modificar les dates, shan de tenir en compte els segents controls:
	- Sha de verificar que la data dinici ha de ser anterior a la data final.
	- Sha de verificar que la data final ha de ser posterior a la data dinici.
	- Si es modificant les dues dates (data inici, data final), sha de verificar que la data final ha de ser posterior a la data dinici.
	- Sha de verificar que la data de publicaci ha de ser com a mnim 7 dies abans que la data dinici de la votaci.

PARAMETRES:
ENTRADA
Obligatoris:
	p_idvotacio	Identificaci de la votaci
Opcionals:
	p_titol	Ttol de la votaci
	p_descripcio	Descripci de la votaci
	p_datainici	Data dinici de la votaci
	p_datafi 	Data fi de la votaci
	p_datapublicaci	Data de publicaci de la votaci
	p_president	Nom del president de la mesa electoral
	p_censglobal	Cens global de la votaci
	p_web	Adrea web opcional
	p_totalciutadans	Total persones amb capacitat de vot
	p_votstotals	Vots totals per votaci
	p_percentatgevotacio	Percentatge de participaci

SORTIDA
Obligatoris:
	RSP	

PRECONDICI: Existeix la votaci a modificar.
POSTCONDICI: Es modifica la votaci.

RETORNA
   SI TOT CORRECTE
	OK si loperaci sha realitzat amb xit
   SI ES PRODUEIX UNA EXCEPCI
	ERROR: Els camps obligatoris no poden ser nul.
	ERROR: La votaci no existeix a la Base de Dades.
	ERROR: La data dinici no ha de ser posterior a la data final.
	ERROR: La data final no pot ser anterior a la data dinici.
	ERROR: La data de publicaci ha de ser, com a mnim, 7 dies abans que la data dinici.
	ERROR: Tipus derror. En qualsevol altre cas.


*******************************************************************************************************/

PROCEDURE PR_MODIFICACIO_VOTACIO (
	p_idvotacio	      IN NUMBER,	
	p_titol 	      IN VARCHAR2,
  p_descripcio	      IN VARCHAR2,
	p_datainici	      IN DATE,
	p_datafi	      IN DATE,
	p_datapublicacio      IN DATE,
	p_president	      IN VARCHAR2,
	p_censglobal	      IN NUMBER,
	p_web		      IN VARCHAR2,
	p_totalciutadans      IN NUMBER,
	p_votstotals	      IN NUMBER,
	p_percentatgevotacio  IN NUMBER,
	RSP		      OUT VARCHAR2
)

IS

E_CAMPS_NUL		EXCEPTION;
E_VOTACIO_NOEXISTEIX	EXCEPTION;
E_DATAINICI_POSTERIOR	EXCEPTION;
E_DATAFINAL_ANTERIOR	EXCEPTION;
E_DATAPUBLICACIO_MIN	EXCEPTION;

existeix	VARCHAR2(1);
p_entrada	VARCHAR2(1000);
dies		NUMBER;
error_num	NUMBER;
error_missatge	VARCHAR2(100);
datainici    	DATE; 
datafi	     	DATE;

BEGIN

   --S'emmagatzema els parmetres d'entrada a la variable p_entrada
 
   p_entrada := 'P_IDVOTACIO: '||p_idvotacio||', P_TITOL: ' ||p_titol||', P_DESCRIPCIO: '||p_descripcio||
		', P_DATAINICI: '||p_datainici||', P_DATAFI: '||p_datafi||', P_DATAPUBLICACIO: '||p_datapublicacio||
		', P_PRESIDENT: '||p_president||', P_CENSGLOBAL: '||p_censglobal||', P_WEB: '||p_web||
		', P_TOTALCIUTADANS: '||p_totalciutadans||', P_VOTSTOTALS: '||p_votstotals||', P_PERCENTATGEVOTACIO: '||p_percentatgevotacio; 

   --Es verifica que tots els camps obligatoris contingin valor

   IF (TRIM(p_idvotacio) IS NULL) THEN
      RAISE E_CAMPS_NUL;
   END IF;

   --Es verifica que la votaci existeix a la Base de Dades
   BEGIN

   SELECT 'Y' INTO existeix 
   FROM VOTACIO 
   WHERE idvotacio = p_idvotacio AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
      RAISE E_VOTACIO_NOEXISTEIX;
   END;

   --Si es vol modificar les dates, shan de tenir en compte els segents controls:
	-- Sha de verificar que la data dinici ha de ser anterior a la data final.
	-- Sha de verificar que la data final ha de ser posterior a la data dinici.
	-- Si es modificant les dues dates (data inici, data final), sha de verificar que la data final ha de ser posterior a la data dinici.
	-- Sha de verificar que la data de publicaci ha de ser com a mnim 7 dies abans que la data dinici de la votaci.

   IF (p_datainici IS NOT NULL) THEN
       SELECT DATAINICI, nvl(DATAFI, TO_DATE('31/12/9999','DD/MM/YYYY'))
         INTO datainici, datafi
         FROM VOTACIO
        WHERE idvotacio = p_idvotacio;

       IF (p_datainici IS NOT NULL AND p_datafi IS NOT NULL) THEN
         IF (p_datainici > p_datafi) THEN
             RAISE E_DATAINICI_POSTERIOR;
         END IF;
       ELSIF (p_datainici IS NOT NULL) THEN
         IF (p_datainici > datafi) THEN
             RAISE E_DATAINICI_POSTERIOR;
         END IF;
       ELSE
         IF (p_datafi < datainici) THEN
             RAISE E_DATAFINAL_ANTERIOR;
         END IF;
       END IF;
     END IF;

   --Es verifica que la data publicaci ha de ser com a mnim 7 dies abans que la data dinici de la votaci

   dies := p_datainici - p_datapublicacio;
   IF (dies < 7) THEN
	RAISE E_DATAPUBLICACIO_MIN;
   END IF;

   --Si es pasa totes les proves de validaci, es realitzar la modificaci a la Base de Dades.

   UPDATE VOTACIO
        SET  titol		= nvl(p_titol, titol),
             descripcio    	= nvl(p_descripcio, descripcio),
             datainici    	= nvl(p_datainici, datainici),
             datafi     	= nvl(p_datafi, datafi),
             datapublicacio     = nvl(p_datapublicacio, datapublicacio),
             president  	= nvl(p_president, president),
             censglobal         = nvl(p_censglobal, censglobal),
             web          	= nvl(p_web, web),
	     totalciutadans	= nvl(p_totalciutadans, totalciutadans),
	     votstotals		= nvl(p_votstotals, votstotals),
	     percentatgevotacio = nvl(p_percentatgevotacio, percentatgevotacio)
     WHERE   idvotacio      	= p_idvotacio;

   RSP := 'OK';

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_VOTACIONS.PR_MODIFICACIO_VOTACIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   --Excepcions
EXCEPTION

 --Els camps obligatoris sn nuls
   WHEN E_CAMPS_NUL THEN RSP:= 'ERROR: : Els camps obligatoris no poden ser nul'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_VOTACIONS.PR_MODIFICACIO_VOTACIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --La votacio no existeix a la Base de Dades
   WHEN E_VOTACIO_NOEXISTEIX THEN RSP:= 'ERROR: : La votaci no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_VOTACIONS.PR_MODIFICACIO_VOTACIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --La data d'inici posterior a la data final
   WHEN E_DATAINICI_POSTERIOR THEN RSP:= 'ERROR: : La data dinici no ha de ser posterior a la data final'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_VOTACIONS.PR_MODIFICACIO_VOTACIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --La data final anterior a la data inicial
   WHEN E_DATAFINAL_ANTERIOR THEN RSP:= 'ERROR: : La data final no pot ser anterior a la data dinici'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_VOTACIONS.PR_MODIFICACIO_VOTACIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --La data publicaci posterior a 7 dies abans que la data inici
   WHEN E_DATAPUBLICACIO_MIN THEN 
	RSP:= 'ERROR: : La data de publicaci ha de ser, com a mnim, 7 dies abans que la data dinici'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_VOTACIONS.PR_MODIFICACIO_VOTACIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT; 

   WHEN OTHERS THEN
   error_num := SQLCODE;
   error_missatge := SUBSTR(SQLERRM, 1, 200);

   RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_VOTACIONS.PR_MODIFICACIO_VOTACIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

END PR_MODIFICACIO_VOTACIO;


/********************************************************
*							*
*		MANTENIMENT OPCIO			*
*							*
********************************************************/


/*******************************************************************************************************

NOM PROCEDIMENT: P_ALTA_OPCIO

FUNCIONALITAT:

Aquest procediment permet donar dalta una nova opci a la Base de dades, si prviament supera els controls de:
	- Tots els camps obligatoris continguin valor.
	- El registre no shagi inserit amb un altre identificador.
	- Existeix la votaci que es passa per parmetre.

PARAMETRES:
ENTRADA
Obligatoris:
	p_idvotacio	Identificador de la votaci
	p_opcio		Nom de lopci

SORTIDA
Obligatoris:
	RSP	

PRECONDICI: No existeix lopci a crear.
POSTCONDICI: Es dona dalta lopci.

RETORNA:
   SI TOT CORRECTE
	OK si loperaci sha realitzat amb xit
   SI ES PRODUEIX UNA EXCEPCI
	ERROR: Lopci ja existeix a la Base de Dades.	
	ERROR: La votaci no existeix a la Base de Dades.
	ERROR: Els camps obligatoris no poden ser nul.
	ERROR: Tipus derror. En qualsevol altre cas.

*******************************************************************************************************/

PROCEDURE PR_ALTA_OPCIO  (
	p_idvotacio 	   IN NUMBER,
  p_opcio		       IN VARCHAR2,
	RSP		           OUT VARCHAR2
)

IS

E_OPCIO_EXISTEIX	EXCEPTION;
E_CAMPS_NUL		EXCEPTION;
E_VOTACIO_NOEXISTEIX	EXCEPTION;

existeix	VARCHAR2(1);
p_entrada	VARCHAR2(1000);
error_num	NUMBER;
error_missatge	VARCHAR2(100);

BEGIN

   --S'emmagatzema els parmetres d'entrada a la variable p_entrada

   p_entrada := 'P_IDVOTACIO: ' ||p_idvotacio||', P_OPCIO: '||p_opcio;

   --Es verifica que tots els camps obligatoris contingin valor

   IF (TRIM(p_idvotacio) IS NULL OR 
	TRIM(p_opcio) IS NULL)
   THEN
      RAISE E_CAMPS_NUL;
   END IF;

   --Es verifica que el registre no s'hagi insertat anteriorment amb altre identificador.

   BEGIN
   
   SELECT 'Y' INTO existeix 
   FROM OPCIO 
   WHERE idvotacio = p_idvotacio AND opcio = p_opcio AND ROWNUM = 1;

   RAISE E_OPCIO_EXISTEIX;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
     NULL;
   END;

   --Es verifica que la votacio que es pasa per parmetre existeix

   BEGIN
   
   SELECT 'Y' INTO existeix 
   FROM VOTACIO 
   WHERE idvotacio = p_idvotacio AND ROWNUM = 1;   

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
     RAISE E_VOTACIO_NOEXISTEIX;
   END;

   --Si es pasa totes les proves de validaci, es realitzar l'inserci del nou registre a la Base de Dades.

   INSERT INTO OPCIO (IDVOTACIO, OPCIO)
   VALUES (p_idvotacio, p_opcio);
	
   RSP := 'OK';

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_VOTACIONS.PR_ALTA_OPCIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   --Excepcions
EXCEPTION

 --L'opci existeix a la Base de Dades
   WHEN E_OPCIO_EXISTEIX THEN RSP:= 'ERROR: : Lopci ja existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_VOTACIONS.PR_ALTA_OPCIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --Els camps obligatoris sn nuls
   WHEN E_CAMPS_NUL THEN RSP:= 'ERROR: : Els camps obligatoris no poden ser nul'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_VOTACIONS.PR_ALTA_OPCIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --La votaci no existeix a la Base de Dades
   WHEN E_VOTACIO_NOEXISTEIX THEN RSP:= 'ERROR: : La votaci no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_VOTACIONS.PR_ALTA_OPCIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   WHEN OTHERS THEN
   error_num := SQLCODE;
   error_missatge := SUBSTR(SQLERRM, 1, 200);

   RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_VOTACIONS.PR_ALTA_OPCIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

END PR_ALTA_OPCIO;


/*******************************************************************************************************

NOM PROCEDIMENT: P_BAIXA_OPCIO

FUNCIONALITAT:

Aquest procediment permet donar de baixa una opci a la Base de dades, si prviament supera els controls de:
	- Tots els camps obligatoris continguin valor.
	- Existeix lopci a la Base de Dades.

PARAMETRES:
ENTRADA
Obligatoris:
	p_idopcio	Identificaci de lopci

SORTIDA
Obligatoris:
	RSP	

PRECONDICI: Existeix lopci a donar de baixa.
POSTCONDICI: Es dona de baixa lopci.

RETORNA:
   SI TOT CORRECTE
	OK si loperaci sha realitzat amb xit
   SI ES PRODUEIX UNA EXCEPCI
	ERROR: Lopci no existeix a la Base de Dades.
	ERROR: Els camps obligatoris no poden ser nul.'
	ERROR: Tipus derror. En qualsevol altre cas.

*******************************************************************************************************/

PROCEDURE PR_BAIXA_OPCIO  (
	p_idopcio   	IN NUMBER,
	RSP		OUT VARCHAR2
)

IS

E_OPCIO_NOEXISTEIX	EXCEPTION;
E_CAMPS_NUL		EXCEPTION;

existeix	VARCHAR2(1);
p_entrada	VARCHAR2(1000);
error_num	NUMBER;
error_missatge	VARCHAR2(100);

BEGIN

   --S'emmagatzema els parmetres d'entrada a la variable p_entrada

   p_entrada := 'P_IDOPCIO: '||p_idopcio;

   --Es verifica que tots els camps obligatoris contingin valor

   IF (TRIM(p_idopcio) IS NULL)
   THEN
      RAISE E_CAMPS_NUL;
   END IF;

   --Es verifica que el registre no s'hagi insertat anteriorment amb altre identificador.

   BEGIN
   
   SELECT 'Y' INTO existeix 
   FROM OPCIO 
   WHERE idopcio = p_idopcio AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
     RAISE E_OPCIO_NOEXISTEIX;
   END;

   --Si es pasa totes les proves de validaci, es realitzar la baixa del registre a la Base de Dades.

   DELETE FROM OPCIO
   WHERE idopcio = p_idopcio;
	
   RSP := 'OK';

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_VOTACIONS.PR_BAIXA_OPCIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   --Excepcions
EXCEPTION

 --L'opci no existeix a la Base de Dades
   WHEN E_OPCIO_NOEXISTEIX THEN RSP:= 'ERROR: : Lopci no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_VOTACIONS.PR_BAIXA_OPCIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --Els camps obligatoris sn nuls
   WHEN E_CAMPS_NUL THEN RSP:= 'ERROR: : Els camps obligatoris no poden ser nul'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_VOTACIONS.PR_BAIXA_OPCIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   WHEN OTHERS THEN
   error_num := SQLCODE;
   error_missatge := SUBSTR(SQLERRM, 1, 200);

   RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_VOTACIONS.PR_BAIXA_OPCIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

END PR_BAIXA_OPCIO;


/*******************************************************************************************************

NOM PROCEDIMENT: P_MODIFICACIO_OPCIO

FUNCIONALITAT:

Aquest procediment permet modificar una opci a la Base de dades, si prviament supera els controls de:
	- Lopci existeix a la Base de Dades.
	- Tots els camps obligatoris continguin valor.
	- Si es vol modificar la votaci, sha de verificar que existeix la votaci a la Base de Dades.

PARAMETRES:
ENTRADA
Obligatoris:
	p_idopcio	Identificaci de lopci
Opcionals:
	p_idvotacio	Identificador de la votaci
	p_opcio		Nom de lopci
	p_votstotals	Nombre de vots de cada opci
	p_percentatge	Percentatge de vots respecte el total

SORTIDA
Obligatoris:
	RSP	

PRECONDICI: Existeix lopci a modificar.
POSTCONDICI: Es modifica lopci.

RETORNA:
   SI TOT CORRECTE
	OK si loperaci sha realitzat amb xit
   SI ES PRODUEIX UNA EXCEPCI
	ERROR: El camps obligatoris no poden ser nul.
	ERROR: Lopci no existeix a la Base de Dades.
	ERROR: La votaci no existeix a la Base de Dades.
	ERROR: Tipus derror. En qualsevol altre cas.

*******************************************************************************************************/

PROCEDURE PR_MODIFICACIO_OPCIO (
	p_idopcio	      IN NUMBER,
	p_idvotacio	      IN NUMBER,	
	p_opcio 	      IN VARCHAR2,
        p_votstotals	      IN NUMBER,
	p_percentatge	      IN NUMBER,
	RSP		      OUT VARCHAR2
)

IS

E_CAMPS_NUL		EXCEPTION;
E_OPCIO_NOEXISTEIX	EXCEPTION;
E_VOTACIO_NOEXISTEIX	EXCEPTION;

existeix	VARCHAR2(1);
p_entrada	VARCHAR2(1000);
error_num	NUMBER;
error_missatge	VARCHAR2(100);

BEGIN

   --S'emmagatzema els parmetres d'entrada a la variable p_entrada
 
   p_entrada := 'P_IDOPCIO: '||p_idopcio||', P_IDVOTACIO: ' ||p_idvotacio||
		', P_OPCIO: '||p_opcio||', P_VOTSTOTALS: '||p_votstotals||
		', P_PERCENTATGE: '||p_percentatge;

   --Es verifica que tots els camps obligatoris contingin valor

   IF (TRIM(p_idopcio) IS NULL) THEN
      RAISE E_CAMPS_NUL;
   END IF;

   --Es verifica que l'opci existeix a la Base de Dades
   BEGIN

   SELECT 'Y' INTO existeix 
   FROM OPCIO
   WHERE idopcio = p_idopcio AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
      RAISE E_OPCIO_NOEXISTEIX;
   END;

   --Si es modifica la votaci, es verifica que la votaci existeix a la Base de Dades
   BEGIN

   SELECT 'Y' INTO existeix 
   FROM VOTACIO 
   WHERE idvotacio = p_idvotacio AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
      RAISE E_VOTACIO_NOEXISTEIX;
   END;

   --Si es pasa totes les proves de validaci, es realitzar l'actualitzaci a la Base de Dades.

   UPDATE OPCIO
        SET  idvotacio		= nvl(p_idvotacio, idvotacio),
             opcio	    	= nvl(p_opcio, opcio),
	     votstotals		= nvl(p_votstotals, votstotals),
             percentatge     	= nvl(p_percentatge, percentatge)
   WHERE   idopcio      	= p_idopcio;

   RSP := 'OK';

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_VOTACIONS.PR_MODIFICACIO_OPCIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   --Excepcions
EXCEPTION

 --Els camps obligatoris sn nuls
   WHEN E_CAMPS_NUL THEN RSP:= 'ERROR: : Els camps obligatoris no poden ser nul'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_VOTACIONS.PR_MODIFICACIO_OPCIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --L'opci no existeix a la Base de Dades
   WHEN E_OPCIO_NOEXISTEIX THEN RSP:= 'ERROR: : Aquesta opci no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_VOTACIONS.PR_MODIFICACIO_OPCIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT; 

 --La votaci no existeix a la Base de Dades
   WHEN E_VOTACIO_NOEXISTEIX THEN RSP:= 'ERROR: : La votaci no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_VOTACIONS.PR_MODIFICACIO_OPCIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT; 

   WHEN OTHERS THEN
   error_num := SQLCODE;
   error_missatge := SUBSTR(SQLERRM, 1, 200);

   RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_VOTACIONS.PR_MODIFICACIO_OPCIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

END PR_MODIFICACIO_OPCIO;

END PKG_VOTACIONS;
/